@@ -1,5 +1,5 @@
-#!/bin/sh
-VER=0.8
+#!/bin/bash
+VER=1.1
 ##################################################################
 # SCRIPTNAME : MSS-SITE
 # AUTHOR     : Turranius (turranius@hotmail.com)
@@ -8,16 +8,44 @@ VER=0.8
 #              userfiles, and is to be run a SITE command from
 #              the glFTPd configuration file.
 #
+#              Install this script before mss-post.sh and make sure
+#              it works first. They work in the same way but mss-post.sh
+#              cant give you any errors on screen. This script can and will.
+#
 # INSTALLATION (added to glftpd.conf)
 #              site_cmd mss EXEC /bin/mss-site.sh
 #              custom-mss   1
 #
 #              Make sure you have the following binaries in glftpds
 #              /bin dir:
-#              dirname, basename, cat, cut, ls, tail, tr, bc
+#              dirname, basename, egrep, cat, cut, ls, tail, tr, bc
 #
 #              Make sure script is executable (chmod 755 or so).
 #
+#   1.1    Add: Just added one line in the help text to show that
+#               issuing "syncgroup" will also sync the ftp-date/groups
+#               files.
+#
+#          Add: Added a WARNING when running this from shell as root.
+#               The actionfiles gets created with root as owner if so and
+#               users have no right to modify them. Bad if you have a dead
+#               slave and the actionfile remains on the hub with bad perms.
+#               If all slaves are up, they'll get deleted anyway by them.
+#
+#   1.0    Add: Added a check that unsets GLROOT if we are not inside
+#               glftpd. Works even if GLROOT is hardset below in this
+#               script.
+#
+#   0.9    Fix: Fixed it so it works on fbsd too. Replaced all
+#               "$( command )" with "`command`"
+#               Also changed to bash instead of sh as interpriter.
+#
+#          Add: Added commands 'syncpasswd' and 'syncgroup' to force
+#               a sync on either of those files.
+#
+#   0.8.1  Fix: Using 'cancel' any other slave but the one defined
+#               first caused a 'slave not defined' error. No biggie.
+#
 #   0.8  - Add: Command backup to tell all slaves to run
 #               mss-core.sh backup
 #          Add: Command dellogs to clear out *.log
@@ -45,6 +73,9 @@ VER=0.8
 ## Otherwise, it will be read from mss-hub.id in the same folder as mss-post.sh
 # GLROOT="/glftpd"
 
+## Ignore this!
+if [ "$GLROOT" ]; then if [ ! -d "$GLROOT" ] && [ "$FLAGS" ]; then unset GLROOT; fi; fi
+
 ## Remove the # from the below line to hard set where mss-hub.conf is.
 ## Otherwise, it will be read from the same folder as mss-post.sh is in.
 # MSSHUBCONFIG=$GLROOT/bin/mss-hub.conf
@@ -55,7 +86,7 @@ VER=0.8
 ##################################################################
 
 if [ -z $GLROOT ]; then
-  IDENTFILE="$( dirname $0 )/mss-hub.id"
+  IDENTFILE="`dirname $0`/mss-hub.id"
   if [ ! -r "$IDENTFILE" ]; then
     echo "GLROOT not set. Either create a file called mss-hub.id and add 'GLROOT=/glftpd'"
     echo "or add that line to mss-hub.conf."
@@ -75,7 +106,7 @@ if [ ! -d "$GLROOT" ]; then
 fi
 
 if [ -z $MSSHUBCONFIG ]; then
-  MSSHUBCONFIG="$( dirname $0 )/mss-hub.conf"
+  MSSHUBCONFIG="`dirname $0`/mss-hub.conf"
 fi
 
 if [ ! -r $MSSHUBCONFIG ]; then
@@ -86,7 +117,7 @@ else
 fi
 
 if [ -z "$SOURCENAME" ]; then
-  IDENTFILE="$( dirname $0 )/mss-hub.id"
+  IDENTFILE="`dirname $0`/mss-hub.id"
   if [ ! -r "$IDENTFILE" ]; then
     echo "SOURCENAME not set. Either create a file called mss-slave.id and add 'SOURCENAME="SlaveName"'"
     echo "or add that line to mss-slave.conf."
@@ -116,6 +147,13 @@ write_action()
 {
     putlog "Writing '$*' to slave(s) actionfile."
     echo "Writing '$*' to slave(s) actionfile."
+    if [ "$GLROOT" ] && [ -z "$FLAGS" ]; then
+      echo " "
+      echo "WARNING: Running this from shell will create actionfiles that users has no access to modify."
+      echo "This will lead to it not working anymore from inside glftpd if you have 'stale' actionfiles"
+      echo "in $GLETC - Make sure they are picked up and deleted by the slaves, or chmod then to 666 yourself."
+    fi
+
     for CURSLAVE in $SLAVES; do
 	echo "$*" >> "$GLETC/$CURSLAVE.actions"
     done
@@ -143,6 +181,10 @@ if [ ! -w "$GLETC" ]; then
   echo "Actions-directory is not writeable."
   exit
 fi
+if [ -z "$GL_VERSION" ]; then
+  GL_VERSION=1
+fi
+
 
 ## Function for showing help.
 proc_help() {
@@ -155,6 +197,10 @@ proc_help() {
     echo "For above commands, { <user1> <user2> }, can be used to trigger"
     echo "multiple people at once."
     echo ""
+    echo "syncpasswd          - Force a sync of the passwd file to all slaves."
+    echo "syncgroup           - Force a sync of the group file to all slaves."
+    echo "                      Will also sync the ftp-data/groups files on gl2+"
+    echo ""
     echo "fetchlogs           - Sends mss logs from slaves."
     echo "integcheck          - Checks integrity of all userfiles"
     echo "                      and reports back to the FETCHLOGSDIR." 
@@ -187,6 +233,7 @@ proc_help() {
     echo ""
     echo "cancel <slave/all>  - Clears out the actionfile(s) incase you made a mistake."
     echo "                      Observe that once the slave start on them, it's too late."
+    echo "                      -> Use no argument to list all defined slaves. <-"
     echo ""
     echo "trim <size>         - Tell the slave to trim its LOG, STATLOG and HIGHDEBUGLOG down to"
     echo "                      the defined MB level, ie 'trim 2' makes then about 2MB large."
@@ -232,26 +279,26 @@ fi
 ## Procedure for checking command for {,} and split up if so.
 ## Also check if '(' or ')' is in it and quit.
 proc_splitusers() {
-  if [ ! -z "$( echo $GLCMDFULL | grep '{' | grep '}' )" ]; then
-    CURUSERS="$( echo $GLCMDFULL | cut -d'{' -f2- | cut -d'}' -f1 )"
-    if [ "$( echo "$GLCMDFULL" | cut -d'}' -f2 | tr '[:lower:]' '[:upper:]' | tr -d ' ' )" == "/FORCE" ]; then
+  if [ ! -z "`echo $GLCMDFULL | grep '{' | grep '}'`" ]; then
+    CURUSERS="`echo $GLCMDFULL | cut -d'{' -f2- | cut -d'}' -f1`"
+    if [ "`echo "$GLCMDFULL" | cut -d'}' -f2 | tr '[:lower:]' '[:upper:]' | tr -d ' '`" == "/FORCE" ]; then
       FORCE="YES"
     fi
   else
     ## Verify that there isnt any single { or } in it."
-    if [ ! -z "$( echo $GLCMDFULL | grep '{' )" ]; then
+    if [ ! -z "`echo $GLCMDFULL | grep '{'`" ]; then
       echo "Error: If '{' is used, it must be ended with a '}'"; exit 0
-    elif [ ! -z "$( echo $GLCMDFULL | grep '}' )" ]; then
+    elif [ ! -z "`echo $GLCMDFULL | grep '}'`" ]; then
       echo "Error: Endpoint '}' found but not startpoint '{'"; exit 0
     ## Verify ( or ) existance. Error if so.
-    elif [ ! -z "$( echo $GLCMDFULL | grep '(' )" ]; then
+    elif [ ! -z "`echo $GLCMDFULL | grep '('`" ]; then
       echo "Error: Char ( not recognised. Did you mean { ?"; exit 0
-    elif [ ! -z "$( echo $GLCMDFULL | grep ')' )" ]; then
+    elif [ ! -z "`echo $GLCMDFULL | grep ')'`" ]; then
       echo "Error: Char ) not recognised. Did you mean ) ?"; exit 0
     else
-      if [ ! -z "$( echo "$GLCMDFULL" | grep -i "/FORCE" )" ]; then
-        if [ ! -z "$( echo "$CURUSER" | grep -i "/FORCE" )" ]; then
-          CURUSER="$( echo $CURUSER | cut -d'/' -f1 )"
+      if [ ! -z "`echo "$GLCMDFULL" | grep -i "/FORCE"`" ]; then
+        if [ ! -z "`echo "$CURUSER" | grep -i "/FORCE"`" ]; then
+          CURUSER="`echo $CURUSER | cut -d'/' -f1`"
         fi
         FORCE="YES"
       fi
@@ -262,13 +309,13 @@ proc_splitusers() {
 
 ## Begin parsing the command (if any)
 GLCMDFULL="$*"
-GLCMD="$( echo "$1" | tr -s ' ' )"
+GLCMD="`echo "$1" | tr -s ' '`"
 if [ ! -z "$2" ]; then
-  CURUSER="$( echo "$2" )"
+  CURUSER="`echo "$2"`"
 fi
 
 ## SYNC
-if [ "$( echo "$GLCMD" | tr '[:lower:]' '[:upper:]' )" == "SYNC" ]; then
+if [ "`echo "$GLCMD" | tr '[:lower:]' '[:upper:]'`" == "SYNC" ]; then
     putlog "Got command: $* from $USER"
     if [ "$CURUSER" = "ALL" -o "$CURUSER" = "all" ]; then
       write_action "SYNC BASIC *"; exit 0
@@ -293,7 +340,7 @@ if [ "$( echo "$GLCMD" | tr '[:lower:]' '[:upper:]' )" == "SYNC" ]; then
     fi
 
 ## STATS
-elif [ "$( echo "$GLCMD" | tr '[:lower:]' '[:upper:]' )" == "STATS" ]; then
+elif [ "`echo "$GLCMD" | tr '[:lower:]' '[:upper:]'`" == "STATS" ]; then
     putlog "Got command: $* from $USER"
     if [ "$CURUSER" = "ALL" -o "$CURUSER" = "all" ]; then
       write_action "SYNC STATS *"; exit 0
@@ -318,7 +365,7 @@ elif [ "$( echo "$GLCMD" | tr '[:lower:]' '[:upper:]' )" == "STATS" ]; then
     fi
 
 ## FULL
-elif [ "$( echo "$GLCMD" | tr '[:lower:]' '[:upper:]' )" == "FULL" ]; then
+elif [ "`echo "$GLCMD" | tr '[:lower:]' '[:upper:]'`" == "FULL" ]; then
     putlog "Got command: $* from $USER"
     if [ "$CURUSER" = "ALL" -o "$CURUSER" = "all" ]; then
       write_action "SYNC FULL *" ; exit 0
@@ -343,22 +390,32 @@ elif [ "$( echo "$GLCMD" | tr '[:lower:]' '[:upper:]' )" == "FULL" ]; then
     fi
 
 ## FETCHLOGS
-elif [ "$( echo "$GLCMD" | tr '[:lower:]' '[:upper:]' )" == "FETCHLOGS" ]; then
+elif [ "`echo "$GLCMD" | tr '[:lower:]' '[:upper:]'`" == "FETCHLOGS" ]; then
     putlog "Got command: '$*' from $USER"
     write_action "FETCHLOG"; exit 0
 
+## SYNCPASSWD
+elif [ "`echo "$GLCMD" | tr '[:lower:]' '[:upper:]'`" == "SYNCPASSWD" ]; then
+    putlog "Got command: '$*' from $USER"
+    write_action "SYNC PASSWD"; exit 0
+
+## SYNCGROUP
+elif [ "`echo "$GLCMD" | tr '[:lower:]' '[:upper:]'`" == "SYNCGROUP" ]; then
+    putlog "Got command: '$*' from $USER"
+    write_action "SYNC GROUP"; exit 0
+
 ## INTEGCHECK
-elif [ "$( echo "$GLCMD" | tr '[:lower:]' '[:upper:]' )" == "INTEGCHECK" ]; then
+elif [ "`echo "$GLCMD" | tr '[:lower:]' '[:upper:]'`" == "INTEGCHECK" ]; then
     putlog "Got command: $* from $USER"
     write_action "INTEGRITY"; exit 0
 
 ## DELREPORTS
-elif [ "$( echo "$GLCMD" | tr '[:lower:]' '[:upper:]' )" == "DELREPORTS" ]; then
+elif [ "`echo "$GLCMD" | tr '[:lower:]' '[:upper:]'`" == "DELREPORTS" ]; then
     putlog "Got command: $* from $USER"
     write_action "DELREPORTS"; exit 0
 
 ## DELLOGS
-elif [ "$( echo "$GLCMD" | tr '[:lower:]' '[:upper:]' )" == "DELLOGS" ]; then
+elif [ "`echo "$GLCMD" | tr '[:lower:]' '[:upper:]'`" == "DELLOGS" ]; then
     putlog "Got command: $* from $USER"
     rm -f $GLROOT$MSSLOGS/*.log
     echo "All logs deleted from $MSSLOGS"
@@ -366,13 +423,13 @@ elif [ "$( echo "$GLCMD" | tr '[:lower:]' '[:upper:]' )" == "DELLOGS" ]; then
     exit 0
 
 ## BACKUP
-elif [ "$( echo "$GLCMD" | tr '[:lower:]' '[:upper:]' )" == "BACKUP" ]; then
+elif [ "`echo "$GLCMD" | tr '[:lower:]' '[:upper:]'`" == "BACKUP" ]; then
     putlog "Got command: $* from $USER"
     write_action "BACKUP"; exit 0
     exit 0
 
 ## LIST
-elif [ "$( echo "$GLCMD" | tr '[:lower:]' '[:upper:]' )" == "LIST" ]; then
+elif [ "`echo "$GLCMD" | tr '[:lower:]' '[:upper:]'`" == "LIST" ]; then
     putlog "Got command: $* from $USER"
     echo "Listing of $GLROOT$MSSLOGS dir:"
     echo "Note that dates are when they were last fetched."
@@ -381,7 +438,7 @@ elif [ "$( echo "$GLCMD" | tr '[:lower:]' '[:upper:]' )" == "LIST" ]; then
     exit 0
 
 ## READLOG
-elif [ "$( echo "$GLCMD" | tr '[:lower:]' '[:upper:]' )" == "READLOG" ]; then
+elif [ "`echo "$GLCMD" | tr '[:lower:]' '[:upper:]'`" == "READLOG" ]; then
     putlog "Got command: $* from $USER"
     FILE="$2"
     if [ -z "$FILE" ]; then
@@ -411,7 +468,7 @@ elif [ "$( echo "$GLCMD" | tr '[:lower:]' '[:upper:]' )" == "READLOG" ]; then
     exit 0
 
 ## READREPORTS
-elif [ "$( echo "$GLCMD" | tr '[:lower:]' '[:upper:]' )" == "READREPORTS" ]; then
+elif [ "`echo "$GLCMD" | tr '[:lower:]' '[:upper:]'`" == "READREPORTS" ]; then
     putlog "Got command: $* from $USER"
     echo "Currently defined slaves: $SLAVES"
     for SLAVE in $SLAVES; do
@@ -428,10 +485,10 @@ elif [ "$( echo "$GLCMD" | tr '[:lower:]' '[:upper:]' )" == "READREPORTS" ]; the
     exit 0
 
 ## READSITELOG
-elif [ "$( echo "$GLCMD" | tr '[:lower:]' '[:upper:]' )" == "READSITELOG" ]; then
+elif [ "`echo "$GLCMD" | tr '[:lower:]' '[:upper:]'`" == "READSITELOG" ]; then
     putlog "Got command: $* from $USER"
     LINES="$2"
-    if [ ! -z "$( echo $LINES | tr -d '[:digit:]' )" ]; then
+    if [ ! -z "`echo $LINES | tr -d '[:digit:]'`" ]; then
       echo "Lines can only be numerical, not $LINES"; exit 0
     fi
     if [ -z "$LINES" ]; then
@@ -444,7 +501,7 @@ elif [ "$( echo "$GLCMD" | tr '[:lower:]' '[:upper:]' )" == "READSITELOG" ]; the
     fi; exit 0
 
 ## READACTION
-elif [ "$( echo "$GLCMD" | tr '[:lower:]' '[:upper:]' )" == "READACTION" ]; then
+elif [ "`echo "$GLCMD" | tr '[:lower:]' '[:upper:]'`" == "READACTION" ]; then
     putlog "Got command: $* from $USER"
     for CURSLAVE in $SLAVES; do
         echo ""
@@ -457,7 +514,7 @@ elif [ "$( echo "$GLCMD" | tr '[:lower:]' '[:upper:]' )" == "READACTION" ]; then
     done; exit 0
 
 ## CANCEL
-elif [ "$( echo "$GLCMD" | tr '[:lower:]' '[:upper:]' )" == "CANCEL" ]; then
+elif [ "`echo "$GLCMD" | tr '[:lower:]' '[:upper:]'`" == "CANCEL" ]; then
     putlog "Got command: $* from $USER"
     SLAVE="$2"
     if [ -z "$SLAVE" ]; then
@@ -465,7 +522,7 @@ elif [ "$( echo "$GLCMD" | tr '[:lower:]' '[:upper:]' )" == "CANCEL" ]; then
       echo "Current slaves are: $SLAVES"
       exit 0
     fi
-    if [ "$( echo "$SLAVE" | tr '[:lower:]' '[:upper:]' )" == "ALL" ]; then
+    if [ "`echo "$SLAVE" | tr '[:lower:]' '[:upper:]'`" == "ALL" ]; then
       for CURSLAVE in $SLAVES; do
         echo "Removing actionfile for $CURSLAVE"
         if [ -e "$GLETC/$CURSLAVE.actions" ]; then
@@ -480,7 +537,7 @@ elif [ "$( echo "$GLCMD" | tr '[:lower:]' '[:upper:]' )" == "CANCEL" ]; then
         fi
       done; exit 0
     else
-      if [ -z "$( echo "$SLAVES" | grep "^$SLAVE" )" ]; then
+      if [ -z "`echo "$SLAVES" | grep -w "$SLAVE"`" ]; then
         echo "$SLAVE is undefined. Valid slaves are $SLAVES."; exit 0
       else
         CURSLAVE="$SLAVE"
@@ -499,7 +556,7 @@ elif [ "$( echo "$GLCMD" | tr '[:lower:]' '[:upper:]' )" == "CANCEL" ]; then
     fi; exit 0
 
 ## TRIM
-elif [ "$( echo "$GLCMD" | tr '[:lower:]' '[:upper:]' )" == "TRIM" ]; then
+elif [ "`echo "$GLCMD" | tr '[:lower:]' '[:upper:]'`" == "TRIM" ]; then
     putlog "Got command: $* from $USER"
     SIZE="$2"
     if [ -z "$SIZE" ]; then
@@ -515,20 +572,20 @@ elif [ "$( echo "$GLCMD" | tr '[:lower:]' '[:upper:]' )" == "TRIM" ]; then
       echo "Do a 'which tr' from shell and copy it to /glftpd/bin/"
       echo "If it is already there, the permissions are wrong cause I cant execute it."; exit 0
     fi
-    if [ ! -z "$( echo "$SIZE" | tr -d '[:digit:]' )" ]; then
-      if [ "$( echo "$SIZE" | tr '[:lower:]' '[:upper:]' )" != "CLEAR" ]; then
+    if [ ! -z "`echo "$SIZE" | tr -d '[:digit:]'`" ]; then
+      if [ "`echo "$SIZE" | tr '[:lower:]' '[:upper:]'`" != "CLEAR" ]; then
         echo "Only use digits in MB parameter!"; exit 0
       fi
     fi
-    if [ "$( echo "$SIZE" | tr '[:lower:]' '[:upper:]' )" != "CLEAR" ]; then
+    if [ "`echo "$SIZE" | tr '[:lower:]' '[:upper:]'`" != "CLEAR" ]; then
       if [ "$SIZE" -lt "1" ]; then
         echo "Size can not be below 1MB!"; exit 0
       fi
     else
-      SIZE="$( echo $SIZE | tr '[:lower:]' '[:upper:]' )"
+      SIZE="`echo $SIZE | tr '[:lower:]' '[:upper:]'`"
     fi
-    if [ "$( echo "$SIZE" | tr '[:lower:]' '[:upper:]' )" != "CLEAR" ]; then
-      SIZE="$( echo "$SIZE * 1024 * 1024" | bc -l | awk -F"." '{print $1}' )"
+    if [ "`echo "$SIZE" | tr '[:lower:]' '[:upper:]'`" != "CLEAR" ]; then
+      SIZE="`echo "$SIZE * 1024 * 1024" | bc -l | awk -F"." '{print $1}'`"
       if [ -z "$SIZE" -o "$SIZE" = " " ]; then
         echo ""
         echo "Error: Either that was not a number or you are missing some libs."
@@ -541,12 +598,12 @@ elif [ "$( echo "$GLCMD" | tr '[:lower:]' '[:upper:]' )" == "TRIM" ]; then
     write_action "TRIM $SIZE"; exit 0
 
 ## COUNTACTION
-elif [ "$( echo "$GLCMD" | tr '[:lower:]' '[:upper:]' )" == "COUNTACTION" ]; then
-  COUNT=$( ls -1 $GLETC/*.actions* 2> /dev/null | wc -l | awk '{ print $1 }' )
+elif [ "`echo "$GLCMD" | tr '[:lower:]' '[:upper:]'`" == "COUNTACTION" ]; then
+  COUNT=`ls -1 $GLETC/*.actions* 2> /dev/null | wc -l | awk '{ print $1 }'`
   echo $COUNT; exit 0
 
 ## REPLLIST
-elif [ "$( echo "$GLCMD" | tr '[:lower:]' '[:upper:]' )" == "REPLLIST" ]; then
+elif [ "`echo "$GLCMD" | tr '[:lower:]' '[:upper:]'`" == "REPLLIST" ]; then
     putlog "Got command: $* from $USER"
     echo "NOTE: Make sure perms are correct before pushing files out to slaves !!"
     echo "Listing of $MSSREPL dir:"
@@ -555,7 +612,7 @@ elif [ "$( echo "$GLCMD" | tr '[:lower:]' '[:upper:]' )" == "REPLLIST" ]; then
     exit 0
 
 ## REPLICATE
-elif [ "$( echo "$GLCMD" | tr '[:lower:]' '[:upper:]' )" == "REPLICATE" ]; then
+elif [ "`echo "$GLCMD" | tr '[:lower:]' '[:upper:]'`" == "REPLICATE" ]; then
     putlog "Got command: $* from $USER"
     FILENAME="$2"
     TOSLAVE="$3"
@@ -565,7 +622,7 @@ elif [ "$( echo "$GLCMD" | tr '[:lower:]' '[:upper:]' )" == "REPLICATE" ]; then
       echo "   ^--------------> - Replicate a script for a slave. Enter -all for all slaves."
       exit 0
     fi
-    if [ -z "$( echo "$SLAVES" | grep -w -- "$TOSLAVE" )" -a "$TOSLAVE" != "-all" ]; then
+    if [ -z "`echo "$SLAVES" | grep -w -- "$TOSLAVE"`" -a "$TOSLAVE" != "-all" ]; then
       echo "$TOSLAVE is undefined in config. Current slaves are: $SLAVES"
       exit 0
     fi
